import json
import openai
from termcolor import colored
import os


# DEFINE YOUR OPENAI API KEY HERE

# openai.api_key = os.environ["OPENAI_API_KEY"]
# openai.api_key = "sk-..."

print(colored("Welcome to the Full Stack Developer Assistant", "green"))
print(colored("!!! DELETE HTML, CSS, JS FILES IF YOU WANT TO START FROM SCRATCH !!!", "red"))
check_input = input("Press any key to continue or type 'exit' to exit: ")
if check_input.strip().lower() == 'exit':
    exit()

model = "gpt-3.5-turbo-0613"

def return_code(model):
    user_input = ''
    print(colored("\nThe following a multiline input, you can put as much detail as you want in here.", "green"))
    print(colored("\nWhat type of website can I code for? (Type 'done' in a new line and press 'enter' when you are done)", "yellow"))
    while True:
        line = input()
        if line.strip().lower() == 'done':
            break
        user_input += line + '\n'
    print(colored("\nStarting to code...", "green"))
    response = openai.ChatCompletion.create(
        model=model,
        # stream= True, 
        messages=[{"role": "system", "content": "Hello, you are an expert full stack developer. You write flawless html, css(style.css), js.You think step by step and make a plan then You write the entire functional code for html, css, js in their entirety and return it as function arguments"},
            {"role": "user", "content": f"{user_input}"}],
        # stream=True,
        functions=[
            {
                "name": "write_html_css_js_to_file",
                "description": "This function takes in html, css, js arguments and writes each to their respective files",
                "parameters": {
                    "type": "object",
                    "properties": {
                        "plan" : {
                            "type": "string",
                            "description": "your detailed plan(in markdown) to solve errors and improve the code to be written to the plan file",
                        },

                        "html": {
                            "type": "string",
                            "description": "The html to be written to the html file",
                        },
                        "css": {
                            "type": "string",
                            "description": "The css to be written to the css file",
                        },
                        "js": {
                            "type": "string",
                            "description": "The js to be written to the js file",
                        },
                    },
                    "required": ["plan","html", "css", "js"],
                            },
            }

        ],
        function_call={"name": "write_html_css_js_to_file"}, # "auto" or "none"
    )

    message = response.choices[0].message
    

    if message.get("function_call"):
        function_args = json.loads(message["function_call"]["arguments"]) 
        return function_args
    
    else:
        print(colored(message["content"], "green"))
    
# if index.html does not exist, 
if not os.path.exists("index.html"):

    function_args = return_code(model)

    def write_html_css_js_to_file(plan, html, css, js):
        with open("plan.md", "w") as f:
            f.write(plan)
        with open("index.html", "w") as f:
            f.write(html)
        with open("style.css", "w") as f:
            f.write(css)
        with open("script.js", "w") as f:
            f.write(js)

        return "Files written successfully"

    write_html_css_js_to_file(**function_args)



x = 1
# how_many_times.. variable set here will determine the number of times the chatbot will run without asking for user feedback
how_many_times_to_run_autonomously_without_user_feedback = 1
retry_due_to_error = False
while True:
    previous_plan = open("plan.md").read()
    html = open("index.html").read()
    css = open("style.css").read()
    js = open("script.js").read()
    if retry_due_to_error:
        user_input = previous_user_input
    # how_many_times.. variable set above will determine the number of times the chatbot will run without asking for user feedback
    elif x % how_many_times_to_run_autonomously_without_user_feedback == 0:
        user_input = ''
        print(colored("\nThe following a multiline input, you can put as much detail as you want in here.", "green"))
        print(colored("\nWhat type of improvements can I code for this website? (Type 'done' in a new line and press 'enter' when you are done)", "yellow"))
        while True:
            line = input()
            if line.strip().lower() == 'done':
                break
            user_input += line + '\n'
        
    else:
        user_input = "fix bugs, make it more beautiful, and add more functionality. add animations, rounded corners, nice colors, light and dark mode functionality. shadows, etc. do not use any image files"
        
    print(colored("\nStarting to code...", "green"))
    print(colored("Improvement attempt number: " + str(x), "magenta"))
    response = openai.ChatCompletion.create(
        model=model,
        messages= [
            {
                "role": "assistant", "content": "Hello, you are an expert full stack developer. You write flawless html, css(style.css), js, and python code. You will be provided with ,previous plan','html', 'css', 'js' files. You are tasked with improving the files by fixing any errors and issues. You will also always strive to make the website more beautiful and dynamic. User may provide you with guidance on what to improve. But even if there is no user input you will always strive to improve the website and make it more beautiful. think step by step and explain your thought process and what you are doing to improve the website. provide both a content response and the function call arguments. Do not remove useful functionality from the website unless it is problematic or instructed by user. your goal is to expand the functionality of the website and make it more beautiful each time."},
                
                {
                "role": "user", "content": f"""
                Previous plan: (keep the good parts and improve the bad parts)

                {previous_plan}


                current html file(use "style.css" as the name)

                {html}

                current css file:

                {css}

                current js file:

                {js}

                user input: 
                {user_input}

                Your detailed plan and thought process:
                my plan is:
                1: ...
                2: ...
                3: ...
                ...
                etc.
                """
            }],
        # stream=True,
        functions=[
            {
                "name": "write_html_css_js_to_file",
                "description": "This function takes in html, css, js arguments and writes each to their respective files",
                "parameters": {
                    "type": "object",
                    "properties": {
                        "plan" : {
                            "type": "string",
                            "description": "your detailed plan(in markdown) to solve errors and improve the code to be written to the plan file",
                        },
                        "html": {
                            "type": "string",
                            "description": "The html to be written to the html file",
                        },
                        "css": {
                            "type": "string",
                            "description": "The css to be written to the css file",
                        },
                        "js": {
                            "type": "string",
                            "description": "The js to be written to the js file",
                        },
                    },

                    "required": ["plan","html", "css", "js"],
                },
            }

        ],
        function_call={"name": "write_html_css_js_to_file"},
    
    
    )
    

    message = response.choices[0].message
    retry_due_to_error = False

    if message.get("content"):
        print(colored(message["content"], "green"))
        
    

    if message.get("function_call"):
        try:
            function_args = json.loads(message["function_call"]["arguments"]) 
        except Exception as e:
            retry_due_to_error = True
            previous_user_input = user_input
            print(colored("Error due to invalid function call arguments... will try again", "red"))
            print(colored(e, "red"))
            # start the loop over again
            continue
    
    else:
        print(colored(message["content"], "green"))
    

    def write_html_css_js_to_file(plan, html, css, js):
        with open("plan.md", "w") as f:
            f.write(plan)
        with open("index.html", "w") as f:
            f.write(html)
        with open("style.css", "w") as f:
            f.write(css)
        with open("script.js", "w") as f:
            f.write(js)

        return "Files written successfully"

    print(write_html_css_js_to_file(**function_args))
    

    x += 1